@@ -4,8 +4,12 @@ All rights reserved.
 
 This source code is licensed under the BSD-style license found in the LICENSE file in the root directory of this source tree.
 """
+import threading
+from .db_thread import DB_Thread
 import os
+import atexit
 from .elarautil import Util
+from .exceptions import InvalidCacheParams
 from .lru import LRU, Cache_obj
 from .status import Status
 
@@ -16,37 +20,44 @@ def is_pos(val):
 
 class Elara:
 
-    from .strings import setnx, append, getset, mget, mset, msetnx, slen
+    from .hashtables import hadd, haddt, hexists, hget, hkeys, hmerge, hnew, hpop, hvals
     from .lists import (
-        lnew,
-        lpush,
-        lextend,
-        lindex,
-        lrange,
-        lrem,
-        lpop,
-        llen,
         lappend,
         lexists,
+        lextend,
+        lindex,
         linsert,
+        llen,
+        lnew,
+        lpop,
+        lpush,
+        lrange,
+        lrem,
     )
-    from .hashtables import hnew, hadd, haddt, hget, hpop, hkeys, hvals, hexists, hmerge
     from .shared import (
-        retmem,
-        retdb,
-        retkey,
         commit,
         exportdb,
         exportkeys,
         exportmem,
+        retdb,
+        retkey,
+        retmem,
         securedb,
         updatekey,
     )
+    from .strings import append, getset, mget, mset, msetnx, setnx, slen
 
     def __init__(self, path, commitdb, key_path=None, cache_param=None):
         self.path = os.path.expanduser(path)
         self.commitdb = commitdb
 
+        # Thread to write into the database
+        self.db_thread = None
+        self.db_lock = threading.Lock()
+
+        # Write data into the database on exit
+        atexit.register(self._autocommit)
+
         if cache_param == None:
             self.lru = LRU()
             self.max_age = None
@@ -57,24 +68,24 @@ class Elara:
                     self.lru = LRU(cache_param["max_size"])
                     self.max_age = cache_param["max_age"]
                 else:
-                    raise Exception
+                    raise InvalidCacheParams("max_age, max_size")
             elif "max_age" in cache_param:
                 if is_pos(cache_param["max_age"]):
                     self.max_age = cache_param["max_age"]
                     self.lru = LRU()
                 else:
-                    raise Exception
+                    raise InvalidCacheParams("max_age")
             elif "max_size" in cache_param:
                 if is_pos(cache_param["max_size"]):
                     self.lru = LRU(cache_param["max_size"])
                     self.max_age = None
                 else:
-                    raise Exception
+                    raise InvalidCacheParams("max_size")
             if "cull_freq" in cache_param:
                 if is_pos(cache_param["cull_freq"]) and cache_param["cull_freq"] <= 100:
                     self.cull_freq = cache_param["cull_freq"]
                 else:
-                    raise Exception
+                    raise InvalidCacheParams("cull_freq")
             else:
                 self.cull_freq = 20
 
@@ -109,10 +120,28 @@ class Elara:
             self.lru._load(self.db, self.max_age)
 
     def _dump(self):
+
         if self.key:
-            Util.encrypt_and_store(self)  # Enclose in try-catch
+            if self.db_thread is not None:
+                self.db_thread.join()
+
+            self.db_thread = DB_Thread(
+                target=Util.encrypt_and_store, args=(self, self.db_lock)
+            )
+            self.db_thread.start()
+            self.db_thread.join()  # Enclose in try-catch
         else:
-            Util.store_plain_db(self)
+
+            if self.db_thread is not None:
+                self.db_thread.join()
+
+            self.db_thread = DB_Thread(
+                target=Util.store_plain_db, args=(self, self.db_lock)
+            )
+            self.db_thread.start()
+            self.db_thread.join()
+
+            # Util.store_plain_db(self)
 
     def _autocommit(self):
         if self.commitdb:
@@ -124,6 +153,19 @@ class Elara:
             del self.db[key]
         self._autocommit()
 
+    # syntax sugar for get, set, rem and exists
+    def __getitem__(self, key):
+        return self.get(key)
+
+    def __setitem__(self, key, value):
+        return self.set(key, value)
+
+    def __delitem__(self, key):
+        return self.rem(key)
+
+    def __contains__(self, key):
+        return self.exists(key)
+
     # Take max_age or self.max_age
     def set(self, key, value, max_age=None):
         if isinstance(key, str):
@@ -135,7 +177,7 @@ class Elara:
                 if is_pos(max_age):
                     cache_obj = Cache_obj(key, max_age)
                 else:
-                    raise Exception
+                    raise InvalidCacheParams("max_age")
 
             # this is for when a key is being overwritten by set
             if self.exists(key):
@@ -165,7 +207,7 @@ class Elara:
             return None
 
     def rem(self, key):
-        if self.lru.rem_key(key) == False:
+        if self.lru.rem_key(key) == Status.NOTFOUND:
             raise Exception
         del self.db[key]
         self._autocommit()
@@ -229,6 +271,15 @@ class Elara:
 
         return len(cache)
 
+    def getmatch(self, match):
+        deleted_keys, cache = self.lru.all()
+        self._remkeys_db_only(deleted_keys)
+        res = {}
+        for key, value in self.db.items():
+            if match in key:
+                res[key] = value
+        return res
+
     def incr(self, key, val=1):
         if self.exists(key):
             data = self.get(key)
